<?php
/**
 * List tables: Templates.
 *
 * @package  Treweler/Admin
 * @version  1.05
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Admin_List_Table_Templates', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Admin_List_Table', false ) ) {
  include_once 'abstract-class-twer-admin-list-table.php';
}

/**
 * TWER_Admin_List_Table_Products Class.
 */
class TWER_Admin_List_Table_Templates extends TWER_Admin_List_Table {

  /**
   * Post type.
   *
   * @var string
   */
  protected $list_table_type = 'twer-templates';

  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();
  }

  /**
   * Define which columns are sortable.
   *
   * @param array $columns Existing columns.
   *
   * @return array
   */
  public function define_sortable_columns( $columns ) {
    $custom = [
      'name'       => 'title',
      'template_id'  => 'template_id',
    ];

    return wp_parse_args( $custom, $columns );
  }

  /**
   * Remove actions from CPT ddl
   *
   * @param $actions
   *
   * @return array
   */
  public function define_bulk_actions( $actions ) {
    unset( $actions['edit'] );

    return $actions;
  }

  /**
   * Define which columns to show on this screen.
   *
   * @param array $columns Existing columns.
   *
   * @return array
   */
  public function define_columns( $columns ) {
    if ( empty( $columns ) && ! is_array( $columns ) ) {
      $columns = [];
    }

    unset( $columns['title'], $columns['comments'], $columns['date'], $columns['categories'] );

    $show_columns = [];
    $show_columns['cb'] = '<input type="checkbox" />';
    $show_columns['name'] = esc_html__( 'Name', 'treweler' );
    $show_columns['template_id'] = esc_html__( 'Template ID', 'treweler' );
    $show_columns['date'] = esc_html__( 'Date', 'treweler' );

    return array_merge( $show_columns, $columns );
  }

  /**
   * Render columm: template_id.
   */
  protected function render_template_id_column() {
    $post_id = isset( $this->object->ID ) ? $this->object->ID : 0;
    echo esc_html( $post_id );
  }


  /**
   * Define primary column.
   *
   * @return string
   */
  protected function get_primary_column() {
    return 'name';
  }

  /**
   * Render column: name.
   */
  protected function render_name_column() {
    global $post;

    $edit_link = get_edit_post_link( $this->object->post_id );
    $title = _draft_or_post_title();

    echo '<strong><a class="row-title" href="' . esc_url( $edit_link ) . '">' . esc_html( $title ) . '</a>';

    _post_states( $post );

    echo '</strong>';

    $parent_id = wp_get_post_parent_id( $this->object->post_id );
    if ( $parent_id > 0 ) {
      echo '&nbsp;&nbsp;&larr; <a href="' . esc_url( get_edit_post_link( $parent_id ) ) . '">' . get_the_title( $parent_id ) . '</a>'; // @codingStandardsIgnoreLine.
    }

    get_inline_data( $post );
  }


  /**
   * Pre-fetch any data for the row each column has access to it. the_marker global is there for bw compat.
   *
   * @param int $post_id Post ID being shown.
   */
  protected function prepare_row_data( $post_id ) {
    global $the_custom_fields;

    if ( empty( $this->object ) || $this->object->post_id !== $post_id ) {
      $the_custom_fields = get_post( $post_id );
      $this->object = $the_custom_fields;
    }
  }

}
