<?php
/**
 * Custom Fields Details
 * Display the custom fields settings details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.05
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Custom_Fields_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once 'abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Custom_Fields_Details Class.
 */
class TWER_Meta_Box_Custom_Fields_Details extends TWER_Meta_Boxes {

  protected $metabox = 'twer-custom-fields';

  protected function set_tabs() {}

  protected function set_fields() {
    $this->fields = [
      [
        'type'     => 'select',
        'name'     => 'custom_field_type',
        'class'    => [ 'js-custom-fields-type' ],
        'label'    => esc_html__( 'Field Type', 'treweler' ),
        'options'  => [
          'text'      => esc_html__( 'Text', 'treweler' ),
          'link'      => esc_html__( 'Link', 'treweler' ),
          'email'     => esc_html__( 'Email', 'treweler' ),
          'phone'     => esc_html__( 'Phone', 'treweler' ),
          'separator' => esc_html__( 'Separator', 'treweler' )
        ],
        'selected' => 'text'
      ],
      [
        'type'  => 'text',
        'name'  => 'custom_field_key',
        'atts'  => [ 'readonly', 'onfocus="this.select();"', 'autocomplete="off"' ],
        'label' => esc_html__( 'Field Key', 'treweler' ),
        'value' => $this->unique_field_key
      ],
      [
        'type'    => 'checkbox',
        'name'    => 'custom_field_show',
        'group_class' => 'form-group col-auto',
        'label'   => esc_html__( 'Show Field Name', 'treweler' ),
        'checked' => false
      ],
      [
        'type'        => 'text',
        'row_id'      => 'custom_field_text',
        'name'        => $this->unique_field_key,
        'atts'        => [ 'disabled' ],
        'group_class' => [ 'col-330' ],
        'label'       => esc_html__( 'Default Value', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_link',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'url_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'url',
            'name'        => 'url',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'URL', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'target',
            'group_class' => 'form-group col-auto',
            'atts'         => [ 'disabled' ],
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'checked'      => false
          ],
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_email',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'email_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'email',
            'name'        => 'email',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Email', 'treweler' ),
            'value'       => '',
          ]
        ]
      ],
      [
        'type'   => 'group',
        'row_id' => 'custom_field_phone',
        'name'   => $this->unique_field_key,
        'label'  => esc_html__( 'Default Value', 'treweler' ),
        'group'  => [
          [
            'type'        => 'text',
            'name'        => 'tel_label',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Text', 'treweler' ),
            'value'       => '',
          ],
          [
            'type'        => 'tel',
            'name'        => 'tel',
            'atts'        => [ 'disabled' ],
            'placeholder' => esc_html__( 'Phone Number', 'treweler' ),
            'value'       => '',
          ]
        ]
      ],
      [
        'type'        => 'number',
        'row_id'      => 'custom_field_separator',
        'name'        => $this->unique_field_key,
        'atts'        => [ 'min="0"', 'step="1"', 'disabled' ],
        'append'      => 'px',
        'placeholder' => '0',
        'group_class' => [ 'col-130' ],
        'label'       => esc_html__( 'Default Value', 'treweler' ),
        'value'       => ''
      ]
    ];
  }


  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();

    add_filter('twer-custom-fields_assign_fields', [$this, 'toggle_rows'], 10, 2);
  }


  public function toggle_rows($fields, $metabox) {
    $meta = twer_get_data();

    $field_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : 'text';

    foreach ( $fields as &$field ) {
      if ( 'separator' === $field_type && $field['id'] === 'custom_field_show' ) {
        $field['row_class'] = implode( ' ', array_merge( [ $field['row_class'] ], [ 'd-none' ] ) );
      }

      if ( $field['id'] === 'custom_field_type' || $field['id'] === 'custom_field_key' || $field['id'] === 'custom_field_show' ) {
        continue;
      }

      if ( 'custom_field_' . $field_type !== $field['id'] ) {
        $field['row_class'] = implode( ' ', array_merge( [ $field['row_class'] ], [ 'd-none' ] ) );
      } else {
        if ( in_array( 'disabled', $field['atts'] ) ) {
          if ( ( $key = array_search( 'disabled', $field['atts'] ) ) !== false ) {
            unset( $field['atts'][ $key ] );
          }
        }

        if('group' === $field['type']) {
          foreach ($field['group'] as $key_group => $group) {
            if ( in_array( 'disabled', $group['atts'] ) ) {
              if ( ( $key = array_search( 'disabled', $group['atts'] ) ) !== false ) {
                unset( $field['group'][$key_group]['atts'][ $key ] );
              }
            }
          }
        }
      }
    }

    return $fields;
  }
}
