<?php
/**
 * Shapes Details
 * Display the shapes details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.10
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Shape_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once 'abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Shape_Details Class.
 */
class TWER_Meta_Box_Shape_Details extends TWER_Meta_Boxes {

  protected function set_tabs() {
    $this->tabs = [
      'styles' => esc_html__( 'Styles', 'treweler' )
    ];
  }

  protected function set_fields() {
    $styles = $this->tabs_mapping( $this->get_styles_fields(), 'styles' );

    $this->fields = $styles;
  }


  public function get_styles_fields() {
    return [
      [
        'type'  => 'message',
        'name'  => 'message',
        'label' => '',
        'style' => 'style="padding: 26px 23px"',
        'value' => esc_html__( 'To edit styles, draw a new shape using the draw tools in the upper right corner of the map or select an existing shape.', 'treweler' )
      ],
      [
        'type'        => 'text',
        'name'        => 'map_lnglat',
        'row_class'   => 'twer-row js-twer-row d-none',
        'label'       => esc_html__( 'Map Coordinates', 'treweler' ),
        'value'       => '[-74.5, 40]'
      ],
      [
        'type'        => 'text',
        'name'        => 'map_zoom',
        'row_class'   => 'twer-row js-twer-row d-none',
        'label'       => esc_html__( 'Map Zoom', 'treweler' ),
        'value'       => '9'
      ],
      [
        'type'        => 'textarea',
        'name'        => 'shape_styles',
        'group_class' => 'form-group col',
        'atts'        => [ 'cols="40"', 'rows="12" ' ],
        'row_class'   => 'twer-row js-twer-row d-none',
        'label'       => esc_html__( 'Styles', 'treweler' ),
        'value'       => ''
      ],


      // Line style settings
      [
        'type'        => 'colorpicker',
        'name'        => 'line_color',
        'row_class'   => 'twer-row js-twer-row d-none',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Line Color', 'treweler' ),
        'value'       => '#438EE4'
      ],
      [
        'type'        => 'number',
        'name'        => 'line_width',
        'row_class'   => 'twer-row js-twer-row d-none',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '2',
        'group_class' => [ 'col-150' ],
        'label'       => esc_html__( 'Line Width', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'      => 'group',
        'name'      => 'line',
        'row_class' => 'twer-row js-twer-row d-none',
        'label'     => esc_html__( 'Dash and Gap', 'treweler' ),
        'group'     => [
          [
            'type'        => 'number',
            'name'        => 'dash',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '1',
            'group_class' => [ 'col-150' ],
            'value'       => ''
          ],
          [
            'type'        => 'number',
            'name'        => 'gap',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '0',
            'group_class' => [ 'col-150' ],
            'value'       => ''
          ],
        ]
      ],


      // Polygon style settings
      [
        'type'        => 'colorpicker',
        'name'        => 'fill_color',
        'row_class'   => 'twer-row js-twer-row d-none',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Fill Color', 'treweler' ),
        'value'       => 'rgba(67,142,228,0.3)'
      ],
      [
        'type'        => 'colorpicker',
        'name'        => 'stroke_color',
        'row_class'   => 'twer-row js-twer-row d-none',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Stroke Color', 'treweler' ),
        'value'       => '#438EE4'
      ],
      [
        'type'        => 'number',
        'name'        => 'stroke_width',
        'row_class'   => 'twer-row js-twer-row d-none',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '2',
        'group_class' => [ 'col-150' ],
        'label'       => esc_html__( 'Stroke Width', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'      => 'group',
        'name'      => 'stroke',
        'row_class' => 'twer-row js-twer-row d-none',
        'label'     => esc_html__( 'Stroke Dash and Gap', 'treweler' ),
        'group'     => [
          [
            'type'        => 'number',
            'name'        => 'dash',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '1',
            'group_class' => [ 'col-150' ],
            'value'       => ''
          ],
          [
            'type'        => 'number',
            'name'        => 'gap',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '0',
            'group_class' => [ 'col-150' ],
            'value'       => ''
          ],
        ]
      ],
    ];
  }

}
