<?php
/**
 * Templates Details
 * Display the templates settings details meta box.
 *
 * @author      Aisconverse
 * @category    Admin
 * @package     Treweler/Admin/Meta Boxes
 * @version     1.05
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( class_exists( 'TWER_Meta_Box_Templates_Details', false ) ) {
  return;
}

if ( ! class_exists( 'TWER_Meta_Boxes', false ) ) {
  include_once 'abstract-class-twer-meta-boxes.php';
}

/**
 * TWER_Meta_Box_Templates_Details Class.
 */
class TWER_Meta_Box_Templates_Details extends TWER_Meta_Boxes {

  protected $metabox = 'twer-templates';

  protected function set_tabs() {
    $this->tabs = [
      'general'    => esc_html__( 'General', 'treweler' ),
      'marker-styles' => esc_html__( 'Marker Styles', 'treweler' ),
      'popup'         => esc_html__( 'Popup', 'treweler' ),
      'label'         => esc_html__( 'Text Label', 'treweler' ),
    ];
    if ( ! empty( $this->get_all_custom_fields() ) ) {
      $this->tabs['custom-fields'] = esc_html__( 'Custom Fields', 'treweler' );
    }
  }

  protected function set_fields() {
    $styles = $this->tabs_mapping( $this->get_styles_fields(), 'marker-styles' );
    $popup = $this->tabs_mapping( $this->get_popup_fields(), 'popup' );
    $general = $this->tabs_mapping( $this->get_general_fields(), 'general' );
    $labels = $this->tabs_mapping( $this->get_label_fields(), 'label' );

    $custom_fields = [];
    if ( ! empty( $this->get_all_custom_fields() ) ) {
      $custom_fields = $this->tabs_mapping( $this->get_custom_fields(), 'custom-fields' );
    }

    $this->fields = array_merge( $general, $styles, $popup, $labels, $custom_fields );
  }

  public function get_all_custom_fields() {
    // Get all custom fields
    return get_posts( [
      'post_status'    => 'publish',
      'post_type'      => 'twer-custom-fields',
      'numberposts'    => - 1,
      'posts_per_page' => - 1,
      'order'          => 'ASC',
    ] );
  }

  public function fill_custom_fields( $includes_custom_fields_posts, $prefix, $meta_data, $hidden = false ) {
    $current_custom_fields = [];
    if ( ! empty( $includes_custom_fields_posts ) ) {
      foreach ( $includes_custom_fields_posts as $field_post ) {
        $field_post_ID = isset( $field_post->ID ) ? $field_post->ID : 0;
        $field_post_title = isset( $field_post->post_title ) ? $field_post->post_title : '';
        $meta = twer_get_data( $field_post_ID );

        $field_post_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : 'text';
        $field_post_key = isset( $meta->custom_field_key ) ? $meta->custom_field_key : 'field-1488228';
        //$field_post_show = isset($meta->custom_field_show) ? $meta->custom_field_show : '';


        $tr_class = ! $hidden ? 'js-ui-slider-item' : 'js-hidden-item d-none';
        $disabled = ! $hidden ? '' : 'disabled';
        switch ( $field_post_type ) {
          case 'separator' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : '';


            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'label'        => $field_post_title,
              'type'         => 'number',
              'append'       => 'px',
              'placeholder'  => '0',
              'group_class'  => [ 'col-130' ],
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'atts'         => [ 'min="0"', 'step="1"', 'disabled' ],
              'name'         => $field_post_key,
              'value'        => $default,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'text' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : '';
            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'label'        => $field_post_title,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'text',
              'group_class'  => [ 'col-330' ],
              'atts'         => [ 'disabled' ],
              'name'         => $field_post_key,
              'value'        => $default,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
          case 'email' :
          case 'link' :
          case 'phone' :
            $default = isset( $meta->{$field_post_key} ) ? $meta->{$field_post_key} : [];

            $text_placeholder = esc_html__( 'URL', 'treweler' );
            $stype = '';
            if ( $field_post_type === 'email' ) {
              $stype = 'email';
              $text_placeholder = esc_html__( 'Email', 'treweler' );
            } elseif ( $field_post_type === 'phone' ) {
              $stype = 'tel';
              $text_placeholder = esc_html__( 'Phone Number', 'treweler' );
            } elseif ( $field_post_type === 'link' ) {
              $stype = 'url';
            }
            $key_default = $stype . '_label';


            $text_default = isset( $default[ $key_default ] ) ? $default[ $key_default ] : '';
            $url_default = isset( $default[ $stype ] ) ? $default[ $stype ] : '';
            $target_default = isset( $default['target'] ) ? $default['target'] : [];
            $target_default = is_array( $target_default ) ? reset( $target_default ) : $target_default;

            $value = [
              [
                'type'        => 'text',
                'name'        => 'text',
                'placeholder' => esc_html__( 'Text', 'treweler' ),
                'atts'        => [ 'disabled' ],
                'value'       => $text_default
              ],
              [
                'type'        => $stype,
                'name'        => 'url',
                'placeholder' => $text_placeholder,
                'atts'        => [ 'disabled' ],
                'value'       => $url_default
              ],
            ];


            if ( $field_post_type === 'link' ) {
              $value[] = [
                'type'         => 'checkbox',
                'name'         => 'target',
                'atts'         => [ 'disabled' ],
                'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
                'checked'      => $target_default
              ];
            }

            $current_custom_fields[] = [
              'row_class'    => 'twer-tr-toggle twer-row js-twer-row ' . $tr_class,
              'row_atts'     => 'data-id="' . $field_post_ID . '"',
              'type'         => 'group',
              'name'         => $field_post_key,
              'label'        => $field_post_title,
              'group'        => $value,
              'row_controls' => '<a href="#" class="js-twer-ui-del-tr twer-ui-del-tr"></a> <a href="#" class="js-twer-ui-sort-tr twer-ui-sort-tr"></a>'
            ];
            break;
        }
      }
    }

    return $current_custom_fields;
  }

  public function get_custom_fields() {
    $meta_data = twer_get_data();
    $prefix = '_treweler_';


    // Get saved custom fields
    $current_custom_fields_ids = isset( $meta_data->custom_fields_list ) ? $meta_data->custom_fields_list : '';

    // Get all includes/saved custom fields and save it to array
    $includes_custom_fields_posts = ! empty( $current_custom_fields_ids ) ? get_posts( [
      'post_status' => 'publish',
      'post_type'   => 'twer-custom-fields',
      'orderby'     => 'post__in',
      'include'     => $current_custom_fields_ids
    ] ) : [];

    $fields_custom = array_merge(
      [
        [
          'row_class' => 'twer-tr-toggle twer-row js-twer-row d-none',
          'label'     => esc_html__( 'Custom Fields', 'treweler' ),
          'type'      => 'text',
          'atts'      => [ 'hidden' ],
          'class'     => [ 'js-twer-custom-fields-list' ],
          'name'      => 'custom_fields_list',
          'value'     => '',
        ],
        [
          'row_class'   => 'twer-tr-toggle twer-row js-twer-row',
          'type'        => 'checkbox',
          'name'        => 'custom_field_show',
          'group_class' => 'form-group col-auto',
          'label'       => esc_html__( 'Show Custom Fields', 'treweler' ),
          'checked'     => false
        ],

        [
          'row_class' => 'twer-tr-toggle twer-row js-twer-row',
          'type'      => 'group',
          'name'      => 'custom_fields_font',
          'label'     => esc_html__( 'Font Style', 'treweler' ),
          'group'     => [
            [
              'type'        => 'number',
              'append'      => 'px',
              'group_class' => [ 'col-130' ],
              'name'        => 'size',
              'atts'        => [ 'min="0"', 'step="1"' ],
              'value'       => '',
              'placeholder' => '13'
            ],
            [
              'type'        => 'select',
              'name'        => 'weight',
              'group_class' => [ 'col-200' ],
              'options'     => $this->font_weight_options,
              'selected'    => '400'
            ],
          ]
        ],
      ],
      $this->fill_custom_fields( $includes_custom_fields_posts, $prefix, $meta_data ),
      $this->fill_custom_fields( $this->get_all_custom_fields(), $prefix, $meta_data, true )
    );


    $select_fields = [
      'none' => __( 'Select a Field', 'treweler' ),
    ];
    // Get all excluded/not saved custom fields and save it to array
    $excluded_custom_fields_posts = get_posts( [
      'post_status'    => 'publish',
      'post_type'      => 'twer-custom-fields',
      'posts_per_page' => - 1,
      'numberposts'    => - 1,
      'exclude'        => $current_custom_fields_ids
    ] );

    foreach ( $excluded_custom_fields_posts as $excluded_custom_field_post ) {
      $id = isset( $excluded_custom_field_post->ID ) ? $excluded_custom_field_post->ID : 0;
      $label = isset( $excluded_custom_field_post->post_title ) ? $excluded_custom_field_post->post_title : '';
      /*$meta = twer_get_data( $id );
      $field_type = isset( $meta->custom_field_type ) ? $meta->custom_field_type : '';*/


      if ( empty( $label ) ) {
        $label = sprintf( __( 'Field: %d' ), $id );
      }
      if ( $id ) {
        $select_fields[ $id ] = $label;
      }
    }

    $fields_custom[] = [
      'type'  => 'group',
      'name'  => 'custom_fields',
      'label' => esc_html__( 'Add fields', 'treweler' ),
      'group' => [
        [
          'type'        => 'select',
          'name'        => 'list',
          'group_class' => [ 'col-200' ],
          'id'          => 'treweler-custom-field-add',
          'options'     => $select_fields,
          'selected'    => 'none',
        ],
        [
          'type'         => 'button',
          'name'         => 'add',
          'group_class'  => 'form-group col-auto',
          'class'        => [ 'js-add-custom-field' ],
          'label_inline' => __( 'Add Field', 'treweler' )
        ],
      ]
    ];

    return $fields_custom;
  }

  public function get_styles_fields() {
    return [
      [
        'type'      => 'select',
        'name'      => 'marker_style',
        'id'        => 'marker_style',
        'row_class' => 'twer-row js-twer-row twer-tr-toggle',
        'label'     => esc_html__( 'Style', 'treweler' ),
        'options'   => [
          'default'          => esc_html__( 'Point', 'treweler' ),
          'dot-default'      => esc_html__( 'Dot', 'treweler' ),
          'balloon-default'  => esc_html__( 'Balloon', 'treweler' ),
          'triangle-default' => esc_html__( 'Triangle', 'treweler' ),
          'custom'           => esc_html__( 'Custom Marker', 'treweler' )
        ],
        'selected'  => 'default'
      ],

      [
        'type'        => 'colorpicker',
        'name'        => 'point_color',
        'label'       => esc_html__( 'Marker Color', 'treweler' ),
        'value'       => '#4b7715',
        'id'          => 'markerColor',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],
      [
        'type'        => 'colorpicker',
        'name'        => 'point_halo_color',
        'label'       => esc_html__( 'Halo Color', 'treweler' ),
        'value'       => '#ffffff',
        'id'          => 'marker_halo_color',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],
      [
        'type'        => 'number',
        'name'        => 'point_halo_opacity',
        'group_class' => 'form-group d-flex align-items-center flex-nowrap col col-242',
        'atts'        => [ 'min="0"', 'step="0.1"', 'max="1"' ],
        'placeholder' => '0.5',
        'id'          => 'treweler-marker-halo-opacity',
        'label'       => esc_html__( 'Halo Opacity', 'treweler' ),
        'value'       => '',
        'help'        => esc_html__( 'The opacity property can take a value from 0 - 1. The lower value, the more transparent.',
          'treweler' ),
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'default' ], 'show' => true ]
      ],

      [
        'type'        => 'checkbox',
        'name'        => 'dot_icon_show',
        'label'       => esc_html__( 'Show Icon', 'treweler' ),
        'checked'     => false,
        'id'          => 'treweler-picker-dot',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'        => 'iconpicker',
        'name'        => 'dot_icon_picker',
        'id'          => 'treweler-dot-icon-picker',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Select Icon', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'treweler-picker-dot', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_icon',
        'label'      => esc_html__( 'Icon', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'dot_icon_color',
            'group_class' => 'form-group col-auto',
            'value'       => '#ffffff',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '15',
            'id'          => 'treweler-dot-icon-size',
            'group_class' => [ 'col-130' ],
            'append'      => 'px',
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'treweler-picker-dot', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot',
        'label'      => esc_html__( 'Marker', 'treweler' ),
        'group'      => [

          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#4b7715',
            'id'          => 'marker_dotcenter_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '12',
            'append'      => 'px',
            'id'          => 'treweler-marker-size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'value'       => '',

          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_border',
        'label'      => esc_html__( 'Border', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'marker_border_color',
            'label'       => esc_html__( 'Border Color', 'treweler' ),
            'value'       => '#ffffff',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '0',
            'append'      => 'px',
            'id'          => 'treweler-marker-border-width',
            'group_class' => [ 'col-130' ],
            'atts'        => [ 'min="0"', 'step="1"' ],
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'dot_corner_radius',
        'label'      => esc_html__( 'Corner Radius', 'treweler' ),
        'group'      => [
          [
            'type'        => 'number',
            'name'        => 'size',
            'id'          => 'treweler-marker-corner-radius-group',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'placeholder' => '50',
            'value'       => '',
          ],
          [
            'type'        => 'select',
            'name'        => 'units',
            'id'          => 'treweler-marker-corner-radius-group-select',
            'group_class' => [ 'col-130' ],
            'options'     => [
              '%'  => esc_html__( '%', 'treweler' ),
              'px' => esc_html__( 'px', 'treweler' ),
            ],
            'selected'    => '%'
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'dot-default' ] ]
      ],


      [
        'type'        => 'checkbox',
        'name'        => 'balloon_icon_show',
        'label'       => esc_html__( 'Show Icon', 'treweler' ),
        'checked'     => false,
        'id'          => 'treweler-picker',
        'group_class' => 'form-group col-auto',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'        => 'iconpicker',
        'name'        => 'balloon_icon_picker',
        'group_class' => 'form-group col-auto',
        'id'          => 'treweler-balloon-icon-picker',
        'label'       => esc_html__( 'Select Icon', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'treweler-picker', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_icon',
        'label'      => esc_html__( 'Icon', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#ffffff',
            'id'          => 'balloon_icon_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '15',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'id'          => 'treweler-balloon-icon-size',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'treweler-picker', 'value' => true ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon',
        'label'      => esc_html__( 'Marker', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'id'          => 'marker_color_balloon',
            'value'       => '#4b7715',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '18',
            'id'          => 'treweler-marker-size-balloon',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'append'      => 'px',
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_border',
        'label'      => esc_html__( 'Border', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#4b7715',
            'id'          => 'marker_border_color_balloon',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'name'        => 'size',
            'placeholder' => '4',
            'append'      => 'px',
            'id'          => 'treweler-marker-border-width-balloon',
            'group_class' => [ 'col-130' ],
            'value'       => '',
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],
      [
        'type'       => 'group',
        'name'       => 'balloon_dot',
        'label'      => esc_html__( 'Dot', 'treweler' ),
        'group'      => [
          [
            'type'        => 'colorpicker',
            'name'        => 'color',
            'value'       => '#ffffff',
            'id'          => 'marker_dot_color',
            'group_class' => 'form-group col-auto',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'placeholder' => '8',
            'id'          => 'treweler-marker-dot-size',
            'append'      => 'px',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
        ],
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'balloon-default' ] ]
      ],


      [
        'type'        => 'colorpicker',
        'name'        => 'triangle_color',
        'id'          => 'marker_color_triangle',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Marker Color', 'treweler' ),
        'value'       => '#4b7715',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],
      [
        'type'        => 'number',
        'name'        => 'triangle_width',
        'placeholder' => '12',
        'id'          => 'treweler-marker-width-triangle',
        'append'      => 'px',
        'label'       => esc_html__( 'Marker Width', 'treweler' ),
        'atts'        => [ 'min="0"', 'step="1"' ],
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],
      [
        'type'        => 'number',
        'name'        => 'triangle_height',
        'placeholder' => '10',
        'append'      => 'px',
        'id'          => 'treweler-marker-height-triangle',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'label'       => esc_html__( 'Marker Height', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'triangle-default' ] ]
      ],


      [
        'type'       => 'image',
        'name'       => 'custom_marker_img',
        'id'         => 'thumbnail_id',
        'label'      => esc_html__( 'Custom Marker', 'treweler' ),
        'value'      => '',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],
      [
        'type'       => 'select',
        'name'       => 'custom_marker_position',
        'label'      => esc_html__( 'Position', 'treweler' ),
        'options'    => $this->position_options,
        'id'         => 'marker_position',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ],
        'selected'   => 'center'
      ],
      [
        'type'        => 'number',
        'name'        => 'custom_marker_size',
        'placeholder' => '42',
        'append'      => 'px',
        'id'          => 'treweler-marker-img-size',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'label'       => esc_html__( 'Marker Size', 'treweler' ),
        'value'       => '',
        'toggle-row'  => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],
      [
        'type'       => 'select',
        'name'       => 'custom_marker_cursor',
        'id'         => 'marker_cursor',
        'label'      => esc_html__( 'Cursor on Hover', 'treweler' ),
        'options'    => [
          'pointer' => esc_html__( 'Pointer', 'treweler' ),
          'grab'    => esc_html__( 'Grab', 'treweler' ),
        ],
        'selected'   => 'pointer',
        'toggle-row' => [ 'related' => 'marker_style', 'value' => [ 'custom' ] ]
      ],

    ];
  }

  public function get_popup_fields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'popup_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show popup', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_heading',
        'label' => esc_html__( 'Heading', 'treweler' ),
        'group' => [
          [
            'type'        => 'text',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '16',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_subheading',
        'label' => esc_html__( 'Subheading', 'treweler' ),
        'group' => [
          [
            'type'        => 'text',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '13',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_description',
        'label' => esc_html__( 'Description', 'treweler' ),
        'group' => [
          [
            'type'        => 'textarea',
            'group_class' => [ 'col-260' ],
            'name'        => 'text',
            'atts'        => [ 'cols="30"', 'rows="6" ' ],
            'value'       => '',
          ],
          [
            'type'        => 'number',
            'name'        => 'size',
            'atts'        => [ 'min="0"', 'step="1"' ],
            'append'      => 'px',
            'placeholder' => '13',
            'group_class' => [ 'col-130' ],
            'value'       => ''
          ],
          [
            'type'        => 'select',
            'name'        => 'weight',
            'group_class' => [ 'col-200' ],
            'options'     => $this->font_weight_options,
            'selected'    => '400'
          ],
        ]
      ],
      [
        'type'        => 'gallery',
        'name'        => 'popup_gallery',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Images', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'popup_gallery_show',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Enable Image Gallery', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_gallery_position',
        'label'    => esc_html__( 'Position', 'treweler' ),
        'id'       => 'treweler-popup-image-position',
        'options'  => [
          'right' => esc_html__( 'Right', 'treweler' ),
          'top'   => esc_html__( 'Top', 'treweler' ),
        ],
        'selected' => 'right'
      ],
      [
        'type'        => 'number',
        'name'        => 'popup_gallery_width',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '160',
        'id'          => 'treweler-popup-image-width',
        'label'       => esc_html__( 'Image width', 'treweler' ),

        'value' => ''
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_button',
        'label' => esc_html__( 'Button', 'treweler' ),
        'group' => [
          [
            'type'         => 'text',
            'name'         => 'text',
            'label_inline' => esc_html__( 'Text', 'treweler' ),
            'value'        => '',
          ],
          [
            'type'         => 'url',
            'name'         => 'url',
            'label_inline' => esc_html__( 'URL', 'treweler' ),
            'value'        => '',
          ],
          [
            'type'         => 'colorpicker',
            'name'         => 'color',
            'group_class'  => 'form-group col-auto d-flex flex-column',
            'label_inline' => esc_html__( 'Color', 'treweler' ),
            'value'        => '#4a7805'
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'target',
            'group_class'  => 'form-group col-auto',
            'label_inline' => esc_html__( 'Open in new tab', 'treweler' ),
            'class'        => [ 'twer-switcher--vertical' ],
            'checked'      => false
          ],
        ]
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_content_align',
        'label'    => esc_html__( 'Content align', 'treweler' ),
        'options'  => [
          'left'   => esc_html__( 'Left', 'treweler' ),
          'center' => esc_html__( 'Center', 'treweler' ),
        ],
        'selected' => 'left'
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_size',
        'label' => esc_html__( 'Min width and height', 'treweler' ),
        'group' => [
          [
            'type'         => 'number',
            'name'         => 'height',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'label_inline' => esc_html__( 'Height', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'width',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'label_inline' => esc_html__( 'Width', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'        => 'number',
        'name'        => 'popup_border_radius',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Corner Radius', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'     => 'select',
        'name'     => 'popup_style',
        'label'    => esc_html__( 'Style', 'treweler' ),
        'options'  => [
          'light' => esc_html__( 'Light', 'treweler' ),
          'dark'  => esc_html__( 'Dark', 'treweler' ),
        ],
        'selected' => 'light'
      ],
      [
        'type'  => 'group',
        'name'  => 'popup_open_on',
        'label' => esc_html__( 'Open popup on', 'treweler' ),
        'group' => [
          [
            'type'         => 'select',
            'name'         => 'action',
            'id'           => 'treweler-popup-open-group-open-on',
            'label_inline' => esc_html__( 'Open popup on', 'treweler' ),
            'options'      => [
              'hover'       => esc_html__( 'Hover', 'treweler' ),
              'click'       => esc_html__( 'Click', 'treweler' ),
              'always_open' => esc_html__( 'Always open', 'treweler' ),
            ],
            'selected'     => 'hover'
          ],
          [
            'type'         => 'checkbox',
            'name'         => 'default',
            'group_class'  => 'form-group col-auto',
            'id'           => 'treweler-popup-open-group-open-default',
            'label_inline' => esc_html__( 'Open by default', 'treweler' ),
            'class'        => [ 'twer-switcher--vertical' ],
            'checked'      => false
          ],
        ]
      ],
    ];
  }

  public function get_general_fields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'marker_enable_clustering',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Enable Clustering', 'treweler' ),
        'checked'     => true
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'marker_enable_center_on_click',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Center On Click', 'treweler' ),
        'checked'     => true
      ]
    ];
  }

  public function get_label_fields() {
    return [
      [
        'type'        => 'checkbox',
        'name'        => 'label_enable',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Show Text Label', 'treweler' ),
        'checked'     => false
      ],
      [
        'type'        => 'textarea',
        'group_class' => [ 'col-445' ],
        'name'        => 'label_description',
        'atts'        => [ 'cols="30"', 'rows="6" ' ],
        'label'       => esc_html__( 'Label Text', 'treweler' ),
        'value'       => ''
      ],
      [
        'row_class'    => 'twer-tr-toggle twer-row js-twer-row',
        'type'        => 'checkbox',
        'name'        => 'label_has_bg',
        'group_class' => 'form-group col-auto',
        'id' => 'treweler-label-has-bg',
        'label'       => esc_html__( 'Show Background', 'treweler' ),
        'checked'     => true,
      ],
      [
        'type'        => 'number',
        'name'        => 'label_border_radius',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Corner Radius', 'treweler' ),
        'value'       => '',
        'toggle-row' => [ 'related' => 'treweler-label-has-bg', 'value' => true ]
      ],
      [
        'type'  => 'group',
        'name'  => 'label_padding',
        'label' => esc_html__( 'Text Padding', 'treweler' ),
        'toggle-row' => [ 'related' => 'treweler-label-has-bg', 'value' => true ],
        'group' => [
          [
            'type'         => 'number',
            'name'         => 'top',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class' => [ 'col-130' ],
            'placeholder' => '8',
            'label_inline' => esc_html__( 'Top', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'bottom',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class' => [ 'col-130' ],
            'placeholder' => '6',
            'label_inline' => esc_html__( 'Bottom', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'left',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class' => [ 'col-130' ],
            'placeholder' => '17',
            'label_inline' => esc_html__( 'Left', 'treweler' ),
            'value'        => ''
          ],
          [
            'type'         => 'number',
            'name'         => 'right',
            'atts'         => [ 'min="0"', 'step="1"' ],
            'append'       => 'px',
            'group_class' => [ 'col-130' ],
            'placeholder' => '17',
            'label_inline' => esc_html__( 'Right', 'treweler' ),
            'value'        => ''
          ],
        ]
      ],
      [
        'type'     => 'select',
        'name'     => 'label_position',
        'label'    => esc_html__( 'Position', 'treweler' ),
        'options'  => $this->position_options,
        'selected' => 'left'
      ],
      [
        'type'        => 'colorpicker',
        'name'        => 'label_color',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Font Color', 'treweler' ),
        'value'       => '#000000'
      ],
      [
        'type'        => 'number',
        'name'        => 'label_size',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '12',
        'label'       => esc_html__( 'Font Size', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'     => 'select',
        'name'     => 'label_weight',
        'label'    => esc_html__( 'Font Weight', 'treweler' ),
        'options'  => $this->font_weight_options,
        'selected' => '400'
      ],
      [
        'type'        => 'number',
        'name'        => 'label_letter_spacing',
        'atts'        => [ 'min="0"', 'step="0.1"' ],
        'append'      => 'px',
        'placeholder' => '0',
        'label'       => esc_html__( 'Letter Spacing', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'number',
        'name'        => 'label_line_height',
        'atts'        => [ 'min="0"', 'step="0.1"' ],
        'placeholder' => '1.3',
        'label'       => esc_html__( 'Line Height', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'number',
        'name'        => 'label_margin',
        'atts'        => [ 'min="0"', 'step="1"' ],
        'append'      => 'px',
        'placeholder' => '5',
        'label'       => esc_html__( 'Margin', 'treweler' ),
        'value'       => ''
      ],
      [
        'type'        => 'checkbox',
        'name'        => 'label_marker_hide',
        'group_class' => 'form-group col-auto',
        'label'       => esc_html__( 'Hide Marker', 'treweler' ),
        'checked'     => false
      ],
    ];
  }


  /**
   * Constructor.
   */
  public function __construct() {
    parent::__construct();

    add_filter( $this->metabox . '_assign_fields', [ $this, 'sort_tab_fields' ], 10, 2 );
  }


  public function sort_tab_fields( $fields, $metabox ) {
    $sorted_fields = [];

    foreach ( $fields as $field ) {
      $sorted_fields[ $field['tab'] ][] = $field;
    }

    return $sorted_fields;
  }

}
